"======================================================================
|
|   Smalltalk TCP/IP sockets - Abstract socket class
|
|   $Revision: 1.95.1$
|   $Date: 2000/12/27 10:45:49$
|   $Author: pb$
|
 ======================================================================"


"======================================================================
|
| Copyright 1988-92, 1994-95, 1999, 2000 Free Software Foundation, Inc.
| Written by Paolo Bonzini.
|
| This file is part of the GNU Smalltalk class library.
|
| The GNU Smalltalk class library is free software; you can redistribute it
| and/or modify it under the terms of the GNU Lesser General Public License
| as published by the Free Software Foundation; either version 2.1, or (at
| your option) any later version.
| 
| The GNU Smalltalk class library is distributed in the hope that it will be
| useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
| General Public License for more details.
| 
| You should have received a copy of the GNU Lesser General Public License
| along with the GNU Smalltalk class library; see the file COPYING.LESSER.
| If not, write to the Free Software Foundation, 59 Temple Place - Suite
| 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================"


Stream subclass: #AbstractSocket
	  instanceVariableNames: 'impl'
	  classVariableNames: 'CheckPeriod Timeout'
	  poolDictionaries: ''
	  category: 'Sockets-Streams'
!

AbstractSocket class instanceVariableNames: 'defaultImplementationClass'!

!AbstractSocket class methodsFor: 'timed-out operations'!

checkPeriod
    "Answer the period that is to elapse between socket polls if data
     data is not ready and the connection is still open (in milliseconds)"
    ^CheckPeriod
!

checkPeriod: anInteger
    "Set the period that is to elapse between socket polls if data
     data is not ready and the connection is still open (in milliseconds)"
    CheckPeriod := anInteger truncated
!

timeout
    "Answer the period that is to elapse between the request for (yet
     unavailable) data and the moment when the connection is considered dead
     (in milliseconds)"
    ^Timeout
!

timeout: anInteger
    "Set the period that is to elapse between the request for (yet
     unavailable) data and the moment when the connection is considered
     dead (in milliseconds)"
    Timeout := anInteger truncated
! !

!AbstractSocket class methodsFor: 'accessing'!

defaultImplementationClass
    "Answer the class that, by default, is used to map between the
     receiver's protocol and a low-level C interface."
    ^defaultImplementationClass
!

defaultImplementationClass: aClass
    "Set which class will be used by default to map between the
     receiver's protocol and a low-level C interface."
    defaultImplementationClass := aClass
! !

!AbstractSocket class methodsFor: 'instance creation'!

new: implementation
    "Answer a new instance of the receiver, using as the underlying
     layer the object passed as the `implementation' parameter; the
     object is probably going to be some kind of AbstractSocketImpl."
    ^super new initialize: implementation
!

new
    "Answer a new instance of the receiver, using a new instance of
     the defaultImplementationClass as the underlying implementation
     layer."
    ^self new: self defaultImplementationClass new
! !

!AbstractSocket methodsFor: 'accessing'!

address
    "Answer an IP address that is of common interest (this can be either
     the local or the remote address, according to the definition in the
     subclass)."
    self subclassResponsibility
!

available
    "Answer whether there is data available on the socket."
    ^self implementation canRead
!

close
    "Close the socket represented by the receiver."
    self flush.
    self implementation close.
!

flush
    "Flush any buffers used by the receiver."
!

isOpen
    "Answer whether the connection between the receiver and the remote
     endpoint is still alive."
    self implementation isNil ifFalse: [ ^false ].
    ^self implementation isOpen
!

localAddress
    "Answer the local IP address of the socket."
    self implementation isNil ifTrue: [ self error: 'socket not connected' ].
    ^self implementation localAddress
!

localPort
    "Answer the local IP port of the socket."
    self implementation isNil ifTrue: [ self error: 'socket not connected' ].
    ^self implementation localPort
!

port
    "Answer an IP port that is of common interest (this can be the port for 
     either the local or remote endpoint, according to the definitions in the
     subclass"
    self subclassResponsibility
!

remoteAddress
    "Answer the IP address of the socket's remote endpoint."
    self implementation isNil ifTrue: [ self error: 'socket not connected' ].
    ^self implementation remoteAddress
!

remotePort
    "Answer the IP port of the socket's remote endpoint."
    self implementation isNil ifTrue: [ self error: 'socket not connected' ].
    ^self implementation remotePort
! !

!AbstractSocket methodsFor: 'printing'!

printOn: aStream
    aStream
	print: self class;
	nextPut: $[;
	print: self address;
	nextPut: $: ;
	print: self port;
	nextPutAll: ']'
! !

!AbstractSocket methodsFor: 'private'!

implementation
    ^impl
!

initialize: implementation
    impl := implementation.
!

waitUntil: aBlock then: resultBlock onTimeoutDo: timeoutBlock

    Timeout // CheckPeriod timesRepeat: [
	aBlock value ifTrue: [ ^resultBlock value ].
	(Delay forMilliseconds: CheckPeriod) wait
    ].
    self close.
    ^timeoutBlock value.
! !

!AbstractSocket methodsFor: 'stream protocol'!

atEnd
    "By default, answer whether the connection is still open."
    ^self isOpen
!

next
    "Read another character from the socket, failing if the connection is
     dead."
    ^self implementation next
!

nextPut: char
    "Write `char' to the socket, failing if the connection is dead.  The
     SIGPIPE signal is automatically caught and ignored by the system."
    ^self implementation nextPut: char
! !

AbstractSocket timeout: 30000; checkPeriod: 1000!