"======================================================================
|
|   AbstractSocketImpl
|
|   $Revision: 1.95.1$
|   $Date: 2000/12/27 10:45:49$
|   $Author: pb$
|
 ======================================================================"


"======================================================================
|
| Copyright 1988-92, 1994-95, 1999, 2000 Free Software Foundation, Inc.
| Written by Paolo Bonzini.
|
| This file is part of the GNU Smalltalk class library.
|
| The GNU Smalltalk class library is free software; you can redistribute it
| and/or modify it under the terms of the GNU Lesser General Public License
| as published by the Free Software Foundation; either version 2.1, or (at
| your option) any later version.
| 
| The GNU Smalltalk class library is distributed in the hope that it will be
| useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
| General Public License for more details.
| 
| You should have received a copy of the GNU Lesser General Public License
| along with the GNU Smalltalk class library; see the file COPYING.LESSER.
| If not, write to the Free Software Foundation, 59 Temple Place - Suite
| 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================"


UnixStream subclass: #AbstractSocketImpl
	   instanceVariableNames: 'localAddress localPort '
	   classVariableNames: ''
	   poolDictionaries: ''
	   category: 'Sockets-Protocols'
!

!AbstractSocketImpl class methodsFor: 'constants'!

soLinger	^128!
soReuseAddr	^4!

sockDgram       ^2!
sockStream      ^1!
!

!AbstractSocketImpl methodsFor: 'abstract'!

bindTo: ipAddress port: port
    "Bind the receiver to the given IP address and port. `Binding' means
     attaching the local endpoint of the socket."
    self subclassResponsibility
!

create
    "Create the underlying socket."
    self subclassResponsibility
!

getSockName
    "Retrieve a ByteArray containing a sockaddr_in struct for the
     local endpoint of the socket."
    self subclassResponsibility
!

optionAt: option level: level size: size
    self subclassResponsibility
!

optionAt: option level: level put: anObject
    self subclassResponsibility
! !

!AbstractSocketImpl methodsFor: 'accessing'!

localAddress
    "Answer the IP address of the local endpoint of the socket."
    ^localAddress
!

localPort
    "Answer the IP port of the local endpoint of the socket."
    ^localPort
! !

!AbstractSocketImpl methodsFor: 'socket options'!

soLinger
    "Answer the number of seconds by which a `close' operation can block
     to ensure that all the packets have reliably reached the destination,
     or nil if those packets are left to their destiny."
    | data |
    data := self
	optionAt: self class soLinger
	level: self class solSocket
	size: CInt sizeof * 2.

    (data intAt: 1) = 0 ifTrue: [ ^nil ].
    ^data intAt: CInt sizeof + 1
!

soLinger: linger
    "Set the number of seconds by which a `close' operation can block
     to ensure that all the packets have reliably reached the destination.
     If linger is nil, those packets are left to their destiny."
    | data |
    data := ByteArray new: CInt sizeof * 2.
    linger isNil ifFalse: [
	data at: 1 put: 1.
	data intAt: CInt sizeof + 1 put: linger
    ].
    self
	optionAt: self class soLinger
	level: self class solSocket
	put: data
!

soReuseAddr
    "Answer whether another socket can be bound the same local address as this
     one.  If you enable this option, you can actually have two sockets with the
     same Internet port number; but the system won't allow you to use the two
     identically-named sockets in a way that would confuse the Internet.  The
     reason for this option is that some higher-level Internet protocols,
     including FTP, require you to keep reusing the same socket number."
    ^((self optionAt: self class soReuseAddr size: CInt sizeof) intAt: 1) > 0
!

soReuseAddr: aBoolean
    "Set whether another socket can be bound the same local address as this one."
    self
	optionAt: self class soReuseAddr
	level: self class solSocket
	put: aBoolean
! !

!AbstractSocketImpl methodsFor: 'private'!

makeByteArray: anObject
    | byteArray |
    (anObject class == ByteArray) ifTrue: [ ^anObject ].
    (anObject class == IPAddress) ifTrue: [ ^anObject asByteArray ].

    byteArray := ByteArray new: CInt sizeof.

    anObject == true	ifTrue: [ byteArray intAt: 1 put: 1 ].
    anObject isInteger	ifTrue: [ byteArray intAt: 1 put: anObject ].
    ^byteArray
!

hasBeenBound
    "Retrieve the local address and port that the receiver has been bound to."
    | port |
    port := ValueHolder new.
    localAddress := IPAddress fromSockAddr: self getSockName port: port.
    localPort := port value
!

nowDead
! !
