"======================================================================
|
|   ServerSocket wrapper class
|
|   $Revision: 1.95.1$
|   $Date: 2000/12/27 10:45:49$
|   $Author: pb$
|
 ======================================================================"


"======================================================================
|
| Copyright 1988-92, 1994-95, 1999, 2000 Free Software Foundation, Inc.
| Written by Paolo Bonzini.
|
| This file is part of the GNU Smalltalk class library.
|
| The GNU Smalltalk class library is free software; you can redistribute it
| and/or modify it under the terms of the GNU Lesser General Public License
| as published by the Free Software Foundation; either version 2.1, or (at
| your option) any later version.
| 
| The GNU Smalltalk class library is distributed in the hope that it will be
| useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
| General Public License for more details.
| 
| You should have received a copy of the GNU Lesser General Public License
| along with the GNU Smalltalk class library; see the file COPYING.LESSER.
| If not, write to the Free Software Foundation, 59 Temple Place - Suite
| 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================"


AbstractSocket subclass: #ServerSocket
	  instanceVariableNames: ''
	  classVariableNames: ''
	  poolDictionaries: ''
	  category: 'Sockets-Streams'
!

!ServerSocket class methodsFor: 'instance creation'!

initialize
    ServerSocket defaultImplementationClass: TCPSocketImpl
! !

!ServerSocket class methodsFor: 'instance creation'!

defaultQueueSize
    "Answer the default length of the queue for pending connections.  When
     the queue fills, new clients attempting to connect fail until the server
     is send #accept to accept a connection from the queue."
    ^50
!

queueSize: backlog
    "Answer a new ServerSocket serving on any local address and port, with a
     pending connections queue of the given length."
    ^self
        port: 0
        queueSize: backlog
        bindTo: IPAddress unknownAddress
!

queueSize: backlog bindTo: ipAddress
    "Answer a new ServerSocket serving on the given local address,
     and on any port, with a pending connections queue of the given length."
    ^self
        port: 0
        queueSize: backlog
        bindTo: ipAddress
!

port: anInteger
    "Answer a new ServerSocket serving on any local address, on the given
     port, with a pending connections queue of the default length."
    ^self
        port: anInteger
        queueSize: self defaultQueueSize
        bindTo: IPAddress unknownAddress
!

port: anInteger queueSize: backlog
    "Answer a new ServerSocket serving on any local address, on the given
     port, with a pending connections queue of the given length."
    ^self
        port: anInteger
        queueSize: backlog
        bindTo: IPAddress unknownAddress
!

port: anInteger bindTo: ipAddress
    "Answer a new ServerSocket serving on the given address and port,
     with a pending connections queue of the default length."
    ^self
        port: anInteger
        queueSize: self defaultQueueSize
        bindTo: ipAddress
!

port: anInteger queueSize: backlog bindTo: ipAddress
    "Answer a new ServerSocket serving on the given address and port,
     and with a pending connections queue of the given length."
    (anInteger < 0) | (anInteger > 16rFFFF)
	ifTrue: [ self error: 'port out of range' ].

    ^self new
	port: anInteger queueSize: backlog bindTo: ipAddress
! !

!ServerSocket methodsFor: 'accessing'!

address
    "Answer the local address"
    ^self localAddress
!

port
    "Answer the local port (the port that the passive socket is listening on)."
    ^self localPort
!

accept
    "Accept a new connection and create a new instance of Socket if there is
     one, else answer nil."
    self available ifFalse: [ ^nil ].	"Make it non-blocking"
    ^self primAccept: Socket
!

accept: socketClass
    "Accept a new connection and create a new instance of socketClass if
     there is one, else answer nil.  This is usually needed only to create
     DatagramSockets."

    self available ifFalse: [ ^nil ].	"Make it non-blocking"
    ^self primAccept: socketClass
!

primAccept: socketClass
    "Accept a new connection and create a new instance of Socket if there is
     one, else fail."
    | implClass newImpl |
    implClass := socketClass defaultImplementationClass.
    newImpl := self implementation accept: implClass.
    ^socketClass new: newImpl
! !

!ServerSocket methodsFor: 'private'!

port: anInteger queueSize: backlog bindTo: ipAddress

    | ipAddressToBindTo |
    ipAddressToBindTo := ipAddress isNil
	ifTrue: [ ipAddress ]
	ifFalse: [ IPAddress unknownAddress ].

    self implementation
	create;
	bindTo: ipAddressToBindTo port: anInteger;
	listen: backlog
! !
